"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vitest_1 = require("vitest");
const documents_1 = require("../../documents");
const CompletionsProvider_1 = require("../CompletionsProvider");
const filters = [
    {
        name: 'upcase',
        syntax: 'string | upcase',
        return_type: [{ type: 'string', name: '' }],
    },
    {
        name: 'downcase',
        syntax: 'string | downcase',
        return_type: [{ type: 'string', name: '' }],
    },
    {
        name: 'split',
        syntax: 'string | split: string',
        return_type: [{ type: 'array', array_value: 'string' }],
    },
    {
        name: 'first',
        syntax: 'array | first',
        return_type: [{ type: 'untyped', name: '' }],
    },
    {
        name: 'map',
        syntax: 'array | map: string',
        return_type: [{ type: 'array', array_value: 'string' }],
    },
    {
        name: 'where',
        syntax: 'array | where: string, string',
        return_type: [{ type: 'array', array_value: 'string' }],
    },
    {
        name: 'metafield_tag',
        syntax: 'metafield | metafield_tag',
        return_type: [{ type: 'string', name: '' }],
    },
    {
        name: 'default',
        syntax: 'variable | default: variable',
        return_type: [{ type: 'untyped', name: '' }],
    },
    {
        name: 'missing_syntax',
        /* syntax: undefined */
    },
];
const objects = [
    {
        name: 'string',
        return_type: [{ type: 'string', name: '' }],
    },
    {
        name: 'number',
        return_type: [{ type: 'number', name: '' }],
    },
    {
        name: 'array',
        return_type: [{ type: 'array', array_value: 'string' }],
    },
    {
        name: 'metafield',
        return_type: [],
    },
    {
        name: 'product',
        return_type: [],
    },
];
const anyFilters = filtersNamesOfInputType('variable').concat(filters.filter((entry) => !entry.syntax).map((x) => x.name));
const stringFilters = filtersNamesOfInputType('string');
const metafieldFilters = filtersNamesOfInputType('metafield');
const arrayFilters = filtersNamesOfInputType('array');
const allFilters = filters.map((x) => x.name).sort();
(0, vitest_1.describe)('Module: FilterCompletionProvider', async () => {
    let provider;
    (0, vitest_1.beforeEach)(async () => {
        provider = new CompletionsProvider_1.CompletionsProvider(new documents_1.DocumentManager(), {
            filters: async () => filters,
            objects: async () => objects,
            tags: async () => [],
        });
    });
    (0, vitest_1.it)('completes with all the filters when no specific filters exist for that type', async () => {
        await (0, vitest_1.expect)(provider).to.complete('{{ product | █ }}', allFilters);
    });
    (0, vitest_1.it)('completes with all the filters when the type is unknown', async () => {
        await (0, vitest_1.expect)(provider).to.complete('{{ undefined | █ }}', allFilters);
    });
    (0, vitest_1.it)('should complete string types with string filters', async () => {
        await (0, vitest_1.expect)(provider).to.complete('{{ string | █ }}', stringFilters.concat(anyFilters));
    });
    (0, vitest_1.it)('should complete array types with array filters', async () => {
        await (0, vitest_1.expect)(provider).to.complete('{{ array | █ }}', arrayFilters.concat(anyFilters));
    });
    (0, vitest_1.it)('should complete named types with named-type filters', async () => {
        await (0, vitest_1.expect)(provider).to.complete('{{ metafield | █ }}', metafieldFilters.concat(anyFilters));
    });
    (0, vitest_1.it)('should infer type in filter chains', async () => {
        await (0, vitest_1.expect)(provider).to.complete('{{ metafield | metafield_tag | █ }}', stringFilters.concat(anyFilters));
        await (0, vitest_1.expect)(provider).to.complete('{{ string | split: "" | █ }}', arrayFilters.concat(anyFilters));
        await (0, vitest_1.expect)(provider).to.complete('{{ array | first | █ }}', allFilters);
    });
    (0, vitest_1.it)('should append the any filters after the filters of the specific type', async () => {
        // As in, the anyFilters are at the _end_ and not shown at the top.
        await (0, vitest_1.expect)(provider).to.complete('{{ string | █ }}', stringFilters.concat(anyFilters));
    });
});
function filtersNamesOfInputType(inputType) {
    return filters
        .filter((x) => { var _a; return (_a = x.syntax) === null || _a === void 0 ? void 0 : _a.startsWith(inputType); })
        .map((x) => x.name)
        .sort();
}
//# sourceMappingURL=FilterCompletionProvider.spec.js.map