"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vitest_1 = require("vitest");
const documents_1 = require("../../documents");
const HoverProvider_1 = require("../HoverProvider");
(0, vitest_1.describe)('Module: LiquidObjectHoverProvider', async () => {
    let provider;
    (0, vitest_1.beforeEach)(async () => {
        provider = new HoverProvider_1.HoverProvider(new documents_1.DocumentManager(), {
            filters: async () => [],
            objects: async () => [
                {
                    name: 'product',
                    description: 'product description',
                    return_type: [],
                    properties: [
                        {
                            name: 'featured_image',
                            return_type: [{ type: 'image', name: '' }],
                        },
                        {
                            name: 'title',
                            return_type: [{ type: 'string', name: '' }],
                        },
                    ],
                },
                {
                    name: 'all_products',
                    return_type: [{ type: 'array', array_value: 'product' }],
                },
                {
                    name: 'paginate',
                    access: { global: false, parents: [], template: [] },
                    return_type: [],
                },
                {
                    name: 'forloop',
                    access: { global: false, parents: [], template: [] },
                    return_type: [],
                },
                {
                    name: 'tablerowloop',
                    access: { global: false, parents: [], template: [] },
                    return_type: [],
                },
                {
                    name: 'image',
                    description: 'image description',
                    access: { global: false, parents: [], template: [] },
                },
                {
                    name: 'section',
                    access: { global: false, parents: [], template: [] },
                },
                {
                    name: 'predictive_search',
                    access: { global: false, parents: [], template: [] },
                },
                {
                    name: 'recommendations',
                    access: { global: false, parents: [], template: [] },
                },
            ],
            tags: async () => [],
        });
    });
    (0, vitest_1.it)('should return the hover description of the object', async () => {
        const contexts = [
            '{{ pro█duct }}',
            '{{ product█ }}',
            '{% echo product█ %}',
            '{% liquid\n echo product█ %}',
            '{% assign x = product %}{{ x█ }}',
            '{% for x in all_products %}{{ x█ }}{% endfor %}',
            '{% assign x = all_products[0] %}{{ x█ }}',
            '{% assign x█ = all_products[0] %}',
            // '{% for x█ in all_products %}{{ x }}{% endfor %}', // not supported yet...
        ];
        for (const context of contexts) {
            await (0, vitest_1.expect)(provider).to.hover(context, vitest_1.expect.stringContaining('product description'));
            await (0, vitest_1.expect)(provider).to.hover(context, vitest_1.expect.stringMatching(/##* \w+: `product`/));
        }
    });
    (0, vitest_1.it)('should support paginate inside paginate tags', async () => {
        const context = `
      {% paginate all_products by 5 %}
        {{ paginate█ }}
      {% endpaginate %}
    `;
        await (0, vitest_1.expect)(provider).to.hover(context, vitest_1.expect.stringMatching(/##* paginate: `paginate`/));
        await (0, vitest_1.expect)(provider).to.hover('{{ paginate█ }}', null);
    });
    (0, vitest_1.it)('should support form inside form tags', async () => {
        const context = `
      {% form all_products by 5 %}
        {{ form█ }}
      {% endform %}
    `;
        await (0, vitest_1.expect)(provider).to.hover(context, vitest_1.expect.stringMatching(/##* form: `form`/));
        await (0, vitest_1.expect)(provider).to.hover('{{ form█ }}', null);
    });
    (0, vitest_1.it)('should support forloop inside for tags', async () => {
        const context = `
      {% for p in all_products %}
        {{ forloop█ }}
      {% endfor %}
    `;
        await (0, vitest_1.expect)(provider).to.hover(context, vitest_1.expect.stringMatching(/##* forloop: `forloop`/));
        await (0, vitest_1.expect)(provider).to.hover('{{ forloop█ }}', null);
    });
    (0, vitest_1.it)('should support tablerowloop inside tablerow tags', async () => {
        const context = `
      {% tablerow p in all_products %}
        {{ tablerowloop█ }}
      {% endtablerow %}
    `;
        await (0, vitest_1.expect)(provider).to.hover(context, vitest_1.expect.stringMatching(/##* tablerowloop: `tablerowloop`/));
        await (0, vitest_1.expect)(provider).to.hover('{{ tablerowloop█ }}', null);
    });
    (0, vitest_1.it)('should support {% layout none %}', async () => {
        await (0, vitest_1.expect)(provider).to.hover(`{% layout none█ %}`, vitest_1.expect.stringMatching(/##* none: `keyword`/));
        await (0, vitest_1.expect)(provider).to.hover('{{ none█ }}', null);
    });
    (0, vitest_1.it)('should support contextual objects by relative path', async () => {
        const contexts = [
            ['section', 'sections/my-section.liquid'],
            ['predictive_search', 'sections/predictive-search.liquid'],
            ['recommendations', 'sections/recommendations.liquid'],
        ];
        for (const [object, relativePath] of contexts) {
            const source = `{{ ${object}█ }}`;
            await (0, vitest_1.expect)(provider).to.hover({ source, relativePath }, vitest_1.expect.stringContaining(`## ${object}`));
            await (0, vitest_1.expect)(provider).to.hover({ source, relativePath: 'file.liquid' }, null);
        }
    });
    (0, vitest_1.it)('should return nothing if the thing is untyped', async () => {
        await (0, vitest_1.expect)(provider).to.hover(`{{ unknown█ }}`, null);
    });
});
//# sourceMappingURL=LiquidObjectHoverProvider.spec.js.map